Imports System.Runtime.InteropServices
Imports System.Windows.Forms
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.Display
Imports ESRI.ArcGIS.esriSystem
Imports ESRI.ArcGIS.Geometry

Public Interface IGPSLayer
	Function GetDisplay() As IDisplay
	Sub SetSymbol(ByVal pMarkerSymbol As IMarkerSymbol)
End Interface

<ComClass(GPSLayerClass.ClassId, GPSLayerClass.InterfaceId, GPSLayerClass.EventsId), _
 ProgId("GPSLayer.GPSLayerClass")> _
Public Class GPSLayerClass
	Implements ESRI.ArcGIS.Carto.ILayer
	Implements IPersistVariant
	Implements IGPSLayer


#Region "COM GUIDs"
	' These  GUIDs provide the COM identity for this class 
	' and its COM interfaces. If you change them, existing 
	' clients will no longer be able to access the class.
	Public Const ClassId As String = "84fa49e4-8567-4825-b52d-c5eafaee8058"
	Public Const InterfaceId As String = "222df29c-a6de-4abf-aa9f-f426a2800592"
	Public Const EventsId As String = "397c9f7f-53db-40d7-b292-b13488a027c9"
#End Region

	Private m_sName As String
	Private m_pDisplay As IDisplay
	Private m_pSR As ISpatialReference
	Private m_bVisible As Boolean
	Private m_pGPSForm As GPSControllerForm
	Private m_pUserMarkerSymbol As IMarkerSymbol
	Private m_pDocMarkerSymbol As IMarkerSymbol

	' A creatable COM class must have a Public Sub New() 
	' with no parameters, otherwise, the class will not be 
	' registered in the COM registry and cannot be created 
	' via CreateObject.
	Public Sub New()
		MyBase.New()

		m_sName = "GPS"
		m_bVisible = False
		m_pGPSForm = Nothing
		GetUserMarkerSymbol()

	End Sub

#Region "IGPSLayer Implementation"
	Public Function GetDisplay() As IDisplay Implements IGPSLayer.GetDisplay
		Return m_pDisplay
	End Function

	Public Sub SetSymbol(ByVal pMarkerSymbol As IMarkerSymbol) Implements IGPSLayer.SetSymbol
		m_pUserMarkerSymbol = pMarkerSymbol
		If m_pUserMarkerSymbol Is Nothing Then
			If Not m_pDocMarkerSymbol Is Nothing Then
				' Reset to document symbol
				m_pGPSForm.SetMarkerSymbol(m_pDocMarkerSymbol)
			End If
		End If
		SetUserMarkerSymbol()
	End Sub
#End Region

#Region "ILayer Implementation"
	Public ReadOnly Property AreaOfInterest() As ESRI.ArcGIS.Geometry.IEnvelope Implements ESRI.ArcGIS.Carto.ILayer.AreaOfInterest
		Get
			If Not m_pDisplay Is Nothing Then
				Return m_pDisplay.DisplayTransformation.VisibleBounds
			End If
			Return New Envelope
		End Get
	End Property

	Public Property Cached() As Boolean Implements ESRI.ArcGIS.Carto.ILayer.Cached
		Get
			Return True
		End Get
		Set(ByVal value As Boolean)
			' Ignore
		End Set
	End Property

	Public Sub Draw(ByVal DrawPhase As ESRI.ArcGIS.esriSystem.esriDrawPhase, ByVal Display As ESRI.ArcGIS.Display.IDisplay, ByVal TrackCancel As ESRI.ArcGIS.esriSystem.ITrackCancel) Implements ESRI.ArcGIS.Carto.ILayer.Draw
		'If Not DrawPhase = esriDrawPhase.esriDPGeography Then
		'   Exit Sub
		'End If
		If Not Display Is m_pDisplay Then
			m_pDisplay = Display
			If Not m_pGPSForm Is Nothing Then
				m_pGPSForm.SetDisplay(Display)
			End If
		End If
	End Sub

	Public Property MaximumScale() As Double Implements ESRI.ArcGIS.Carto.ILayer.MaximumScale
		Get
			Return 0
		End Get
		Set(ByVal value As Double)
			' Ignore
		End Set
	End Property

	Public Property MinimumScale() As Double Implements ESRI.ArcGIS.Carto.ILayer.MinimumScale
		Get
			Return 0
		End Get
		Set(ByVal value As Double)
			' Ignore
		End Set
	End Property

	Public Property Name() As String Implements ESRI.ArcGIS.Carto.ILayer.Name
		Get
			Return m_sName
		End Get
		Set(ByVal value As String)
			m_sName = value
		End Set
	End Property

	Public Property ShowTips() As Boolean Implements ESRI.ArcGIS.Carto.ILayer.ShowTips
		Get
			Return False
		End Get
		Set(ByVal value As Boolean)
			' Ignored
		End Set
	End Property

	Public WriteOnly Property SpatialReference() As ESRI.ArcGIS.Geometry.ISpatialReference Implements ESRI.ArcGIS.Carto.ILayer.SpatialReference
		Set(ByVal value As ESRI.ArcGIS.Geometry.ISpatialReference)
			m_pSR = value
			If Not m_pGPSForm Is Nothing Then
				m_pGPSForm.SetSpatialRef(value)
			End If
		End Set
	End Property

	Public ReadOnly Property SupportedDrawPhases() As Integer Implements ESRI.ArcGIS.Carto.ILayer.SupportedDrawPhases
		Get
			Return esriDrawPhase.esriDPGeography
		End Get
	End Property

	Public ReadOnly Property TipText(ByVal x As Double, ByVal y As Double, ByVal Tolerance As Double) As String Implements ESRI.ArcGIS.Carto.ILayer.TipText
		Get
			Return Nothing
		End Get
	End Property

	Public ReadOnly Property Valid() As Boolean Implements ESRI.ArcGIS.Carto.ILayer.Valid
		Get
			Return True
		End Get
	End Property

	Public Property Visible() As Boolean Implements ESRI.ArcGIS.Carto.ILayer.Visible

		Get
			Return m_bVisible
		End Get
		Set(ByVal value As Boolean)

			If (value) Then

				' Create GPS form, initialize it, and show it

				Dim bCreate As Boolean = False
				If m_pGPSForm Is Nothing Then
					bCreate = True
				ElseIf Not m_pGPSForm.Created Then
					m_pGPSForm = Nothing
					GC.Collect()
					bCreate = True
				End If
				If bCreate Then
					m_pGPSForm = New GPSControllerForm
					m_pGPSForm.SetLayer(Me)
				End If
				If Not m_pUserMarkerSymbol Is Nothing Then
					m_pGPSForm.SetMarkerSymbol(m_pUserMarkerSymbol)
				ElseIf Not m_pDocMarkerSymbol Is Nothing Then
					m_pGPSForm.SetMarkerSymbol(m_pDocMarkerSymbol)
				End If
				If Not m_pDisplay Is Nothing Then
					m_pGPSForm.SetDisplay(m_pDisplay)
				End If
				m_pGPSForm.Show()

			Else

				' Shut down everything and destroy GPS form

				If Not m_pGPSForm Is Nothing Then
					m_pGPSForm.ShutDown()
					If m_pGPSForm.Visible Then
						m_pGPSForm.Hide()
					End If
					m_pGPSForm = Nothing
					GC.Collect()
				End If

			End If

			m_bVisible = value

		End Set

	End Property

#End Region

#Region "IPersistVariant Implementation"

	Public ReadOnly Property ID() As UID Implements IPersistVariant.ID
		Get
			Dim uid As UID = New UIDClass()
			uid.Value = "GPSLayer.GPSLayerClass"
			Return uid
		End Get
	End Property

	Public Sub Load(ByVal Stream As IVariantStream) Implements IPersistVariant.Load
		Dim pMarkerSymbol As IMarkerSymbol = Nothing
		Try
			pMarkerSymbol = Stream.Read()
		Catch ex As Exception
			pMarkerSymbol = Nothing
		End Try
		If Not pMarkerSymbol Is Nothing Then
			m_pDocMarkerSymbol = pMarkerSymbol
		End If
		Marshal.ReleaseComObject(Stream)
	End Sub

	Public Sub Save(ByVal Stream As IVariantStream) Implements IPersistVariant.Save
		If m_pUserMarkerSymbol Is Nothing Then
			Exit Sub
		End If
		Try
			Stream.Write(m_pUserMarkerSymbol)
		Catch ex As Exception
			' Do nothing
		End Try
		Marshal.ReleaseComObject(Stream)
	End Sub
#End Region

#Region "User settings"
	Private Sub SetUserMarkerSymbol()
		If m_pUserMarkerSymbol Is Nothing Then
			My.Settings.MarkerSymbol = ""
			My.Settings.Save()
		End If
		Try
			Dim pWrapper As IXMLPersistedObject = New XMLPersistedObject
			pWrapper.Object = m_pUserMarkerSymbol
			Dim pSerializer As IXMLSerializer = New XMLSerializer
			Dim sSetting As String = pSerializer.SaveToString(pWrapper, Nothing, Nothing)
			My.Settings.MarkerSymbol = sSetting
			My.Settings.Save()
		Catch ex As Exception

		End Try
	End Sub

	Private Sub GetUserMarkerSymbol()
		Dim sSetting As String = My.Settings.MarkerSymbol
		If sSetting Is Nothing Or sSetting = "" Then
			Exit Sub
		End If
		Try
			Dim pSerializer As IXMLSerializer = New XMLSerializer
			Dim pWrapper As IXMLPersistedObject = pSerializer.LoadFromString(sSetting, Nothing, Nothing)
			m_pUserMarkerSymbol = pWrapper.Object
		Catch ex As Exception

		End Try
	End Sub
#End Region

End Class


